package com.wanji.indicators.event.stream;

import com.wanji.indicators.event.cross.conflictpoint.ConflictEventMain;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.source.ConsulConfigSource;
import com.wanji.indicators.source.debug.DebugSourceCarFrame;
import com.wanji.indicators.util.CommonUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 离线计算入口  com.mapabc.project.event.cross.stream.OfflineTestMain
 * @author ke.han
 */
public class OfflineTestMain {

    private static final Logger log = LoggerFactory.getLogger(OfflineTestMain.class);

    public static void main(String[] args) {
        //初始化环境
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        env.setParallelism(1);

        //加载配置项 并获取工作路径
        String path = new CommonUtil().getWorkSpasePath();
        String outPath = "d:/flink/out/";
        String osName = System.getProperties().getProperty("os.name");
        if (StringUtils.isNotBlank(osName) && osName.contains("Windows")) {
            //此处是 Windows 配置
        } else {
            path = "/opt/flink/workspace/";
        }

        //读取离线数据
        DataStreamSource<CrossFrameModel> streamSource = env.addSource(new DebugSourceCarFrame(path));

        SingleOutputStreamOperator<CrossFrameModel> thinningStream = streamSource
                .keyBy(CrossFrameModel::getCrossId)
                .countWindow(1)
                .maxBy("globalTimeStamp");

        //离线调试模式
        boolean isDebug = true;
        ConflictEventMain.init(env,outPath,isDebug).run(thinningStream);

        try {
            env.execute("事故事件测试");
        } catch (Exception e) {
            log.error("事件测试 - 计算任务异常 : " + e);
        }
    }
}
