package com.wanji.indicators.event.stream;

import com.wanji.indicators.event.cross.conflictpoint.ConflictEventMain;
import com.wanji.indicators.event.stream.func.CrossFrameFlatMap;
import com.wanji.indicators.model.CrossFrameModel;
import com.wanji.indicators.util.PropertiesHelper;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.connector.kafka.source.reader.deserializer.KafkaRecordDeserializationSchema;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.kafka.clients.consumer.OffsetResetStrategy;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

public class TrafficEventMain {
    //private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);

        private static final Logger log = LoggerFactory.getLogger(TrafficEventMain.class);

        public static void main(String[] args) {

            //初始化配置文件
            PropertiesHelper instance = PropertiesHelper.getInstance();
            Properties properties = instance.getProperties();

            //初始化环境
            final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
            env.setParallelism(1);

            String topic = properties.getProperty("consumer.topic");

            KafkaSource<String> source = KafkaSource.<String>builder()
                    .setProperties(instance.getConsumerProperties())
                    .setProperty("auto.offset.commit", "true")
                    .setProperty("auto.commit.interval.ms", "1000")
                    .setProperty("commit.offsets.on.checkpoint", "true")
                    .setBootstrapServers(properties.getProperty("bootstrap.servers"))
                    .setTopics(topic)
                    .setGroupId(properties.getProperty("consumer.group.id")+"-event")
                    /* 设置起始偏移量有以下几种情况
                           1.从指定的位置消费：OffsetsInitializer.offsets(Map<TopicPartition, Long> offsets)
                           2.从最新位置消费(最后一条处)：OffsetsInitializer.latest()
                           3.从最早位置消费(第一条处)：OffsetsInitializer.earliest()
                           4.从上次提交的位置消费：OffsetsInitializer.committedOffsets()
                           5.新的组，从来没有提交过,再指定一个消费方式：OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST)
                          */
                    .setStartingOffsets(OffsetsInitializer.committedOffsets(OffsetResetStrategy.LATEST))
                    // 从大于等于此时间戳开始的偏移量开始
                    //.setStartingOffsets(OffsetsInitializer.timestamp(dateTime.getMillis()))

                    .setDeserializer(KafkaRecordDeserializationSchema.valueOnly(StringDeserializer.class))

                    .build();

            DataStream<String> streamSource = env
                    .fromSource(source, WatermarkStrategy.noWatermarks(), "kafka-car-track-source-event");

            SingleOutputStreamOperator<CrossFrameModel> thinningDataStream =
                    streamSource
                            .flatMap(new CrossFrameFlatMap())
                            .setParallelism(1)
                            .name("全域轨迹帧数据-按路口分组输出");

            //抽帧
            SingleOutputStreamOperator<CrossFrameModel> thinningStream = thinningDataStream
                    .keyBy(CrossFrameModel::getCrossId)
                    .countWindow(3)
                    .maxBy("globalTimeStamp")
                    .name("数据抽帧");

            //路口内冲突点计算
            ConflictEventMain.init(env).run(thinningStream);


            try {
                env.execute("全域安全专题计算");
            } catch (Exception e) {
                log.error("全域安全专题计算任务异常 :   " + e);
            }
        }

}
