package com.wanji.indicators.source;

import com.wanji.indicators.constant.Constant;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.source.RichParallelSourceFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;

/**
 * 查询 rid 转向数据 对应的冲突转向
 */
public class ConflictTurnSource extends RichParallelSourceFunction<Map<String, List<Map<String, Object>>>> {

    private static final Logger log = LoggerFactory.getLogger(ConflictTurnSource.class);

    private volatile boolean running = true;

    public void open(Configuration parameters) throws Exception {
    }

    public static void main(String[] args) {
        String s = getQuerySql();
        System.out.println(s);
    }

    private static String getQuerySql() {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("SELECT t1.inter_id,t1.f_rid,t1.t_rid,t1.turn_dir_no as f_turn_dir_no,t2.clash_f_rid,t2.turn_dir_no as t_turn_dir_no ");
        stringBuilder.append("FROM ( ");
        stringBuilder.append("SELECT b.inter_id,a.f_rid,a.t_rid,b.turn_dir_no,a.clash_f_rid,a.clash_t_rid,a.rr_id,a.clash_rr_id ");
        stringBuilder.append("FROM gis_turn_clash a  JOIN cross_relation b  ");
        stringBuilder.append("on a.rr_id=b.rr_id and b.turn_dir_no !=4 ");
        stringBuilder.append(") t1 JOIN ( ");
        stringBuilder.append("SELECT b.inter_id,a.f_rid,a.t_rid,b.turn_dir_no,a.clash_f_rid,a.clash_t_rid,a.rr_id,a.clash_rr_id ");
        stringBuilder.append("FROM gis_turn_clash a  JOIN cross_relation b ");
        stringBuilder.append("on a.clash_rr_id=b.rr_id and a.f_rid != a.clash_f_rid and b.turn_dir_no !=4 ");
        stringBuilder.append(") t2 ON t1.f_rid = t2.f_rid and t1.t_rid=t2.t_rid and t1.clash_f_rid=t2.clash_f_rid and t1.clash_t_rid=t2.clash_t_rid  ");
        stringBuilder.append("AND t1.inter_id=t2.inter_id ");
        stringBuilder.append("JOIN (SELECT DISTINCT inter_id from lane_obj where data_source='2' and inter_id is not null) t3 ");
        stringBuilder.append("ON t1.inter_id=t3.inter_id and t2.inter_id=t3.inter_id");
        return stringBuilder.toString();
    }

    @Override
    public void run(SourceContext<Map<String, List<Map<String, Object>>>> sourceContext) throws Exception {
        //key:rid->turn value:冲突rid及转向
        Map<String, List<Map<String, Object>>> retMap = new HashMap<>();
        try {
            List<HashMap> dataList = null;//new JdbcUtil().excuteQueryToList(HashMap.class, getQuerySql());
            if (dataList != null) {
                retMap.clear();
                for (HashMap<String, Object> map : dataList) {
                    String crossId = map.get("inter_id").toString();
                    String rid = map.get("f_rid").toString();
                    String f_turn_dir_no = map.get("f_turn_dir_no").toString();
                    String clash_f_rid = map.get("clash_f_rid").toString();//冲突方向rid
                    String t_turn_dir_no = map.get("t_turn_dir_no").toString();//冲突方向转向

                    String key = rid + Constant.MARK + f_turn_dir_no + Constant.MARK + crossId;
                    List<Map<String, Object>> tmplist = retMap.get(key);
                    if (Objects.isNull(tmplist)) {
                        tmplist = new ArrayList<>();
                    }

                    Map<String, Object> conflictTurnMap = new HashMap<>();
                    conflictTurnMap.put("f_rid", clash_f_rid);
                    conflictTurnMap.put("turn_dir_no", t_turn_dir_no);

                    tmplist.add(conflictTurnMap);

                    retMap.put(key, tmplist);

                    sourceContext.collect(retMap);
                }
            }
        } catch (Exception e) {
            log.error("查询 rid 冲突信息数据异常: " + e.getMessage(), e);
        }
    }

    @Override
    public void cancel() {
        running = false;
    }

    /**
     * 理论冲突相位数据，
     * 相位标号定义 以北方向为基准定义 1:北直行相位 2：东直行相位 3：南直行相位 4：西直行相位 5：北左转相位 6：东左转 7：南左转 8：西左转
     */
    public Map<Integer, List<Integer>> createConflictSlots() {

        Map<Integer, List<Integer>> maplist = new HashMap<Integer, List<Integer>>();

        for (int i = 1; i < 9; i++) {
            Integer sidewalkDir = null;
            List<Integer> conflictList = maplist.get(i);
            if (Objects.isNull(conflictList)) {
                conflictList = new ArrayList<>();
            }
            if (i < 5) { // 直行相位
                // 直行相位的冲突相位点
                // 顺时针直行相位
                int clockwise = (i + 1) % 4 == 0 ? 4 : (i + 1) % 4;
                // 逆时针直行相位
                int antiClockWise = (i - 1) % 4 == 0 ? 4 : (i - 1) % 4;
                // 对向左转冲突点
                int acrossLeft = ((i + 2) % 4 == 0 ? 4 : (i + 2) % 4) + 4;
                // 逆时针左转冲突相位点
                int antiLeft = ((i - 1) % 4 == 0 ? 4 : (i - 1) % 4) + 4;

                conflictList.add(clockwise);
                conflictList.add(antiClockWise);
                conflictList.add(acrossLeft);
                conflictList.add(antiLeft);
            } else {// 左转相位冲突相位

                // 顺时针直行相位
                int antiClockWiseStraight = (i + 1) % 4 == 0 ? 4 : (i + 1) % 4;
                // 对向直行冲突点
                int acrossStraight = (i + 2) % 4 == 0 ? 4 : (i + 2) % 4;
                // 顺时针左转相位
                int clockwiseLeft = ((i + 1) % 4 == 0 ? 4 : (i + 1) % 4) + 4;
                // 逆时针左转相位
                int antiClockWiseLeft = ((i - 1) % 4 == 0 ? 4 : (i - 1) % 4) + 4;

                conflictList.add(antiClockWiseStraight);
                conflictList.add(acrossStraight);
                conflictList.add(clockwiseLeft);
                conflictList.add(antiClockWiseLeft);
            }
            maplist.put(i, conflictList);
        }

        return maplist;

    }

//    private static String getQuerySql1() {
//        StringBuilder stringBuilder = new StringBuilder();
//        stringBuilder.append("SELECT a.inter_id,a.f_rid,a.turn_dir_no,b.dir_4_no, ");
//        stringBuilder.append("( ");
//        stringBuilder.append("CASE ");
//        stringBuilder.append("WHEN b.dir_4_no=3 and turn_dir_no=2 THEN 1 ");
//        stringBuilder.append("WHEN b.dir_4_no=3 and turn_dir_no=1 THEN 5 ");
//        stringBuilder.append("WHEN b.dir_4_no=4 and turn_dir_no=2 THEN 2 ");
//        stringBuilder.append("WHEN b.dir_4_no=4 and turn_dir_no=1 THEN 6 ");
//        stringBuilder.append("WHEN b.dir_4_no=1 and turn_dir_no=2 THEN 3 ");
//        stringBuilder.append("WHEN b.dir_4_no=1 and turn_dir_no=1 THEN 7 ");
//        stringBuilder.append("WHEN b.dir_4_no=2 and turn_dir_no=2 THEN 4 ");
//        stringBuilder.append("WHEN b.dir_4_no=2 and turn_dir_no=1 THEN 8 ");
//        stringBuilder.append("END ");
//        stringBuilder.append(") as slot_no ");
//        stringBuilder.append("FROM cross_relation a ");
//        stringBuilder.append("JOIN rid b on a.f_rid=b.rid ");
//        stringBuilder.append("WHERE  turn_dir_no not in (3,4)");
//        return stringBuilder.toString();
//    }

//    public void run1(SourceContext<Map<String, List<Map<String, Object>>>> sourceContext) throws Exception {
//        //key:rid->turn value:冲突rid及转向
//        Map<String, List<Map<String, Object>>> retMap = new HashMap<>();
//        try {
//            List<HashMap> dataList = new JdbcUtil().excuteQueryToList(HashMap.class, getQuerySql1());
//            Map<Integer, List<Integer>> baseConflictInfo = createConflictSlots();
//
//            if (dataList != null && !dataList.isEmpty()) {
//                retMap.clear();
//                for (HashMap<String, Object> map : dataList) {
//                    String inter_id = map.get("inter_id").toString();
//                    String rid = map.get("f_rid").toString();
//                    String turn_dir_no = map.get("turn_dir_no").toString();
//
//                    //当前相位编号
//                    Integer slot_no = Integer.valueOf(map.get("slot_no").toString());
//                    //冲突相位
//                    List<Integer> conflictSlot = baseConflictInfo.get(slot_no);
//                    //冲突相位关联rid等信息
//                    List<HashMap<String, Object>> conflictSlotList = dataList.stream().filter(m ->Objects.equals(inter_id,m.get("inter_id"))  && conflictSlot.contains(Integer.valueOf(m.get("slot_no").toString()))).collect(Collectors.toList());
//                    if (!conflictSlotList.isEmpty()) {
//                        String key = rid + Constant.MARK + turn_dir_no;
//                        List<Map<String, Object>> tmplist = retMap.get(key);
//                        if (Objects.isNull(tmplist)) {
//                            tmplist = new ArrayList<>();
//                        }
//                        tmplist.addAll(conflictSlotList);
//                        retMap.put(key, tmplist);
//                        sourceContext.collect(retMap);
//                    }
//                }
//
//
//            }
//        } catch (Exception e) {
//            log.error("查询 rid 冲突信息数据异常: " + e.getMessage(), e);
//        }
//    }

}
