package com.wanji.indicators.task.track.service.func;

import com.wanji.indicators.model.CarTrackModel;
import com.wanji.indicators.model.FrameModel;
import com.wanji.indicators.model.RoutePathCarTrackModel;
import com.wanji.indicators.util.CarNumGenerator;
import com.wanji.indicators.util.DateUtil;
import org.apache.flink.api.common.functions.FlatMapFunction;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.util.Collector;
import org.apache.kafka.common.protocol.types.Field;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 帧数据转换
 */
public class CarTrackFlatMap implements FlatMapFunction<FrameModel, CarTrackModel> {

    private static final Logger log = LoggerFactory.getLogger(CarTrackFlatMap.class);
    //孪生出的数据暂时没有车牌号，通过ID映射一个随机车牌号作为测试
    static Map<Integer,String[]> randomCarNumMap = new ConcurrentHashMap<>();
    static  String motorObjectType = "1,2,3,7,8,10,11,12,13,15";
    boolean isMockPlate = false;

    @Override
    public void flatMap(FrameModel frameModel, Collector<CarTrackModel> collector) throws Exception {

        String timeStamp = frameModel.getTimeStamp();
        Long globalTimeStamp = frameModel.getGlobalTimeStamp();
        Integer participantNum = frameModel.getParticipantNum();
        String globalId = frameModel.getOrgCode();

        List<CarTrackModel> list = frameModel.getTrackList();

        String[] sps = motorObjectType.split(",");
        List<String> motorTypeList = Arrays.asList(sps);

        list.forEach(o->{
            String type = o.getOriginalType().toString();
            o.setTimeStamp(DateUtil.toDateTime(globalTimeStamp,"yyyy-MM-dd HH:mm:ss.SSS"));
            o.setGlobalTimeStamp(globalTimeStamp);
            o.setOrgCode(globalId);
            collector.collect(o);
        });
    }

    /**
     * 获取随机数
     *
     * @param min
     * @param max
     * @return
     */
    public static int getRandom(int min, int max) {
        int floor = (int) Math.floor(Math.random() * (max - min + 1) + min);
        return floor;
    }

}
